#include "Gadget/Engine.h"

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	Core G;
	G.Window.SetSize(1024, 768);
	G.Window.SetFullScreen(true);
	G.Window.Open();
	G.Mouse.SetVisibility(false);

	//Notice that OpenGL is completely exposed so that OpenGL functions can be used. This mean that you can extend
	//the capabilities of the engine by writing your own OpenGL code or by incorporating APIs that use OpenGL.
	glDisable(GL_DEPTH_TEST);
	//I have turned off the depth testing. This means I'll have to render the objects in the scene from back to front.

	//Add a space background
	Image starsImg; starsImg.Load("Images/stars.jpg"); //Load the image from disk.

	Sprite starsSpr(float(G.Window.GetWidth()), float(G.Window.GetHeight())); //Stretch the background to fit the screen
	//Now place the sprite in the center of the screen
	starsSpr.x = float(G.Window.GetWidth())/2.0f;
	starsSpr.y = float(G.Window.GetHeight())/2.0f;

	//Add a spaceship
	Image shipImg; shipImg.Load("Images/ship1.jpg"); //Load the image from disk.
	/*
	Have a look at the Ship.jpg picture. It's a cheesy looking green spaceship but there's a problem with 
	it. It's a JPG file, and JPGs don't have transparency like PNGs can. So the spaceship has a black 
	background which would block the stars. Change the blend mode to 0 to see that in action. To get rid 
	of the black background I'll set the blend mode to 3 which is additive transparency. The dark portions 
	of the picture are transparent, and the light portions are used to brighten or add to whatever was 
	drawn behind it. That is why the order that we render things with transparency matters.
	*/
	
	Sprite shipSpr(300.0f, 300.0f);
	shipSpr.x = 200.0f;
	shipSpr.y = 200.0f;

	/*
	Our animating explosion has four frames of animation. Each frame is a separate JPG file. So 
	we'll create an array of 4 images to hold each of the frames.
	*/
	Image explosionImg[4]; 
	explosionImg[0].Load("Images/explode1.jpg"); //Load frame 1
	explosionImg[1].Load("Images/explode2.jpg"); //Load frame 2
	explosionImg[2].Load("Images/explode3.jpg"); //Load frame 3
	explosionImg[3].Load("Images/explode4.jpg"); //Load frame 4

	Sprite explosionSpr(300.0f, 300.0f);
	
	float timer = 0.0f; //A variable to keep track of which frame we are displaying

	while(G.Window.IsOpen())//loop as long as the window is open
	{
		G.Renderer.BeginFrame(); G.Renderer.Render2D();

			//ESC closes the window
			if(G.Keyboard.GetKeyHold(GLFW_KEY_ESC)) {G.Window.Close();}	

			//Render the ship
			Shader::Blend(0);
			starsImg.Activate();
			starsSpr.Render();

			//Rotate and render the spaceship
			Shader::Blend(3);
			shipImg.Activate();
			shipSpr.rotation += 1.0f;
			shipSpr.Render();

			//Render the animating explosion
			timer += G.Timer.GetElapsedTime()*4.0f; //advance the timer
			if(timer >= 4.0f) 
			{
				//When it reaches the end of the animation, move the explosion to a random location on screen and
				//start at the beginning.
				explosionSpr.x = float(rand()%G.Window.GetWidth());
				explosionSpr.y = float(rand()%G.Window.GetHeight());
				timer = 0.0f;
			}
			/*
			Our timer counts from 0 to 4 every second so the animation should take one second to 
			play through.
			*/

			//I set the blend mode to 3 for the spaceship. The explosion should also use this mode, so I don't need
			//to set it again.
			explosionImg[int(timer)].Activate();//Activate one of the four images
			explosionSpr.Render();

			/*
			It's always a good idea to turn off your shaders and blending before refreshing the screen.
			Shaders could cause problems for some video cards or make the game run slow. We can turn off any 
			active shaders by calling the Shader class's Deactivate method.
			*/

			Shader::Deactivate();
			
		G.Renderer.EndFrame();	
	}
	return 0;
}			